#define XEMPORTS    0xC02
#define XEPORTS     0xC22

#define MAX_ALLOC   0x100

#define MAXBOARDS   12
#define FEPCODESEG  0x0200L
#define FEPCODE     0x2000L
#define BIOSCODE    0xf800L

#define MISCGLOBAL  0x0C00L
#define NPORT       0x0C22L
#define MBOX        0x0C40L
#define PORTBASE    0x0C90L

/* Begin defines for use with epca_paranoia() */

#define	CHECKFORFEP	1
#define CHECKIFBDVALID	2

/* Begin code defines used for epca_setup */

#define INVALID_BOARD_TYPE   0x1
#define INVALID_NUM_PORTS    0x2
#define INVALID_MEM_BASE     0x4
#define INVALID_PORT_BASE    0x8
#define INVALID_BOARD_STATUS 0x10
#define INVALID_ALTPIN       0x20

/* End code defines used for epca_setup */


#define FEPCLR      0x00
#define FEPMEM      0x02
#define FEPRST      0x04
#define FEPINT      0x08
#define	FEPMASK     0x0e
#define	FEPWIN      0x80

#define PCXE    0
#define PCXEVE  1	/* No longer used */
#define PCXEM   2
#define EISAXEM 3	/* No longer used */
#define PC64XE  4	/* No longer used */
#define PCXI    5
#define PCXR    6       // PEB added Dec '97
#define ISACX   7       // PEB added Dec '98
#define SPXX    8       // PEB added July '99	/* No longer used */
#define EISACX  9       // PEB added Jan '99	/* No longer used */
#define NONE1	10	// CWS added for compat Feb '00
#define NONE2	11	// CWS added for compat Feb '00
#define NONE3	12	// CWS added for compat Feb '00
#define NONE4	13	// CWS added for compat Feb '00
#define NONE5	14	// CWS added for compat Feb '00
#define PCIXEM  15
#define PCICX   16
#define PCIXR   17
#define PCIXRJ  18
#define PCI920_4  19    // PEB added Dec '97
#define PCI920_8  20    // PEB added Dec '97
#define PCI920_2  21    // PEB added Jan '99
#define PCIXR_422  22   // PEB added Mar '99
#define PCIEPCJ 23      // CWS added May '99	/* No longer used */
#define EPCA_NUM_TYPES 24

struct {
	unsigned short dpatype;
	char *desc;
} btypes[] = {
	{ (T_PCXE | T_PCXX | T_ISABUS),    "Acceleport Xe ISA"  	},
	{ 0, 				   "" 				},  /* Used to be Xeve ISA  */
	{ (T_PCXEM | T_PCLITE | T_ISABUS), "Acceleport Xem ISA" 	},
	{ 0, 				   "" 				},  /* Used to be Xem EISA  */
	{ 0, 				   "" 				},  /* Used to be Xe64k ISA */
	{ (T_PCXI | T_PCXX | T_ISABUS),	   "Acceleport Xi ISA"		}, 
	{ (T_PCXR | T_PCLITE | T_ISABUS),  "Acceleport Xr ISA" 		},
	{ (T_CX | T_ISABUS),		   "Acceleport C/X ISA"		},
	{ 0, 				   "" 				},  /* Used to be Smartport Plus */
	{ 0, 				   "" 				},  /* Used to be C/X EISA    */
	// For future ISA board expansion - CWS
	{ 0,				   ""				},
	{ 0,				   ""				},
	{ 0, 				   ""				},
	{ 0,				   ""				},
	{ 0,				   ""				},
	
	{ (T_PCXEM | T_PCLITE | T_PCIBUS), "Acceleport Xem PCI" 	},
	{ (T_CX | T_PCIBUS),		   "Acceleport C/X PCI" 	},
	{ (T_PCXR | T_PCLITE | T_PCIBUS),  "Acceleport Xr PCI"  	},
	{ (T_PCXR | T_PCLITE | T_PCIBUS),  "Acceleport Xr(PLX) PCI" 	},
	{ (T_PCXR | T_PCLITE | T_PCIBUS),  "Acceleport 4r-920 PCI" 	},
	{ (T_PCXR | T_PCLITE | T_PCIBUS),  "Acceleport 8r-920 PCI" 	},
	{ (T_PCXR | T_PCLITE | T_PCIBUS),  "Acceleport 2r-920 PCI" 	},
	{ (T_PCXR | T_PCLITE | T_PCIBUS),  "Acceleport Xr-422" 		},
	{ (T_EPC  | T_PCIBUS),		   "Acceleport EPC/X PCI"	},
	{ 0, 				   ""  				} /* Used to be EPC/X(PLX) PCI */
}; 
                                                                                                                                                        
#define STARTC      021
#define STOPC       023

/* Enable auxilary start/stop output */
#define IAIXON      0x2000
/* Swaps DCD, DSR on concentrator */
#define IALTPIN 0x0080


#define TXSTOPPED  0x1
#define LOWWAIT    0x2
#define EMPTYWAIT  0x4
#define RXSTOPPED  0x8
#define TXBUSY     0x10

#define DISABLED   0
#define ENABLED    1
#define OFF        0
#define ON         1

#define FEPTIMEOUT 200000  
#define SERIAL_TYPE_NORMAL  1
#define SERIAL_TYPE_CALLOUT 2
#define SERIAL_TYPE_INFO    3
#define EPCA_EVENT_HANGUP   1
#define EPCA_MAGIC          0x5c6df104L

/* CX Concentrator download request */
#define DOWNREQ 0x0D00

struct channel 
{
	long   magic;
	unchar boardnum;
	unchar channelnum;
	unchar omodem;         /* FEP output modem status     */
	unchar imodem;         /* FEP input modem status      */
	unchar modemfake;      /* Modem values to be forced   */
	unchar modem;          /* Force values                */
	unchar hflow;
	unchar dsr;
	unchar dcd;
	unchar printer;		/* is this channel a printer? 1=yes/0=no */
	unchar m_rts ; 		/* The bits used in whatever FEP */
	unchar m_dcd ;		/* is indiginous to this board to */
	unchar m_dsr ;		/* represent each of the physical */
	unchar m_cts ;		/* handshake lines */
	unchar m_ri ;
	unchar m_dtr ;
	unchar stopc;
	unchar startc;
	unchar stopca;
	unchar startca;
	unchar fepstopc;
	unchar fepstartc;
	unchar fepstopca;
	unchar fepstartca;
	unchar txwin;
	unchar rxwin;
	ushort fepiflag;
	ushort fepcflag;
	ushort fepoflag;
	ushort txbufhead;
	ushort txbufsize;
	ushort rxbufhead;
	ushort rxbufsize;
	int    close_delay;
	int    count;
	int    blocked_open;
	int    event;
	int    asyncflags;
	uint   dev;
	long   session;
	long   pgrp;
	ulong  statusflags;
	ulong  c_iflag;
	ulong  c_cflag;
	ulong  c_lflag;
	ulong  c_oflag;
	unchar *txptr;
	unchar *rxptr;
	unchar *tmp_buf;
	struct board_info           *board;
	volatile struct board_chan  *brdchan;
	struct digi_struct          digiext;
	struct tty_struct           *tty;
	struct termios              normal_termios;
	struct termios              callout_termios;
#if LINUX_VERSION_CODE > 0x020300
	wait_queue_head_t           open_wait;
	wait_queue_head_t           close_wait;
#else
	struct wait_queue           *open_wait;
	struct wait_queue           *close_wait;
#endif
	struct tq_struct            *tqueue;
	volatile struct global_data *mailbox;
};

struct board_info	
{
	unchar status;
	int type;
	unchar altpin;
	ushort numports;
	unsigned long port;
	ulong  membase;
	unchar *re_map_port;
	unchar *re_map_membase;
	ulong  memory_seg;
	ushort memkb;
	ulong winsize;		/* How big is the window? */
	ulong winbits;		/* How many bits to shift */
	void ( * memwinon )	(struct board_info *, unsigned int) ;
	void ( * memwinoff ) 	(struct board_info *, unsigned int) ;
	void ( * globalwinon )	(struct channel *) ;
	void ( * txwinon ) 	(struct channel *) ;
	void ( * rxwinon )	(struct channel *) ;
	void ( * memoff )	(struct channel *) ;
	void ( * assertgwinon )	(struct channel *) ;
	void ( * assertmemoff )	(struct channel *) ;
	unchar poller_inhibited ;
	unchar initted ;
	ushort firstchan;
	char dpastatus;
	struct pci_dev *pdev;	/* Store this board's PCI pointer */
	char used;		/* This is for pci_boards[]. it allows us */
				/* to know whether we've "found" this one */
				/* in a PCI detection yet		  */
#if LINUX_VERSION_CODE > 0x020300
        wait_queue_head_t           kme_wait;
#else
	struct wait_queue           *kme_wait;
#endif
};

#if LINUX_VERSION_CODE < 0x020300
# if !defined(DECLARE_WAITQUEUE)
#  define DECLARE_WAITQUEUE(w,c)  struct wait_queue w = {(c), NULL}
# endif
#endif

typedef unsigned char   int_8;
typedef unsigned short  int_16;
typedef unsigned int    int_32;

#define DOWNREQ 0x0D00
//  Download buffer structure(s).
// The header info.
typedef struct {
  int_8 type; // Header
  int_8 seq;  // Download sequence
  int_16 srev; // Software revision number
  int_16 lrev; // Low revision number
  int_16 hrev; // High revision number
  int_16 seg;  // Start segment address
  int_16 size; // Number of bytes to download
} DLH ;

// On the Host Adapter, header is followd by data area.
typedef struct {
  DLH   hdr;
  int_8 data[1024]; // Download data
} DL_CX;


// Used to copy the C/X concentrator download image from digiDload to driver
#define CXCONC_CHUNKSIZE 1024
typedef struct {
	unsigned int size;
	unsigned char data[CXCONC_CHUNKSIZE];
} CXCONC_UPLOAD;

#define EPCXCONC_CHUNKSIZE 1024
typedef struct {
	unsigned int size;
	unsigned char data[EPCXCONC_CHUNKSIZE];
} EPCXCONC_UPLOAD;


/* Allocate space for this many ports by default (unless overridden by
 * a runtime reinitialization of this parameter)
 */
#define MAXPORTS 256
/* NDEV_MAJOR is number of devices supported per Linux major number */
#define NDEV_MAJOR 256

/* Each bank of NDEV_MAJOR devices requires a letter identifier
 * (e.g: the 'D' in /dev/ttyD").  Since some letters are already reserved
 * (e.g: "ttyS*" is reserved for the internal serial ports), the available
 * letters are listed in order in TTY_BANKNAMES */

#ifdef CONFIG_DEVFS_FS
# define TTY_NAME "dg/epca/%d"
#else
# define TTY_NAME "ttyD"
#endif

/* Skip 'G' -- ttyG* reserved for digi dgdm driver    */
/* Skip 'S' -- ttyS* reserved for built-in comm ports */
#define TTY_BASENAME "tty"
#define TTY_BANKNAMES "DEFHIJKLMNOPQRTUVWXYZ"



/*
 * Driver identification, error and debugging statments
 *
 * In theory, you can change all occurances of "epca" in the next
 * three lines, and the driver printk's will all automagically change.
 *
 * APR((fmt, args, ...));       Always prints message
 * DPR((fmt, args, ...));       Only prints if EPCA_TRACER is defined at
 *                                compile time and dgdm_debug!=0
 */
#define DRVSTR          "epca"                  /* Driver name string
                                                 * displayed by APR      */
 
#define APR(args)       do { TRC(args); printk(DRVSTR": "); printk args; \
                           } while (0)
#define RAPR(args)      do { TRC(args); printk args; } while (0)



/*
 * We don't actually have any tracing at this point.  stub it out.
 */
# define TRC(ARGS)

